@@ -23,6 +23,7 @@ import gtk
 import pango
 
 from rednotebook.util import markup
+from rednotebook.util import utils
 from rednotebook import undo
 
 
@@ -34,7 +35,8 @@ class CategoriesTreeView(object):
         self.undo_redo_manager = main_window.undo_redo_manager
 
         # Maintain a list of all entered categories. Initialized by rn.__init__()
-        self.categories = None
+        self.categories = []
+        self.last_category = ''
 
         self.statusbar = self.main_window.statusbar
 
@@ -47,7 +49,7 @@ class CategoriesTreeView(object):
         # create the TreeViewColumn to display the data
         self.tvcolumn = gtk.TreeViewColumn()
         label = gtk.Label()
-        label.set_markup('<b>' + _('Categories') + '</b>')
+        label.set_markup('<b>' + _('Tags') + '</b>')
         label.show()
         self.tvcolumn.set_widget(label)
 
@@ -64,8 +66,8 @@ class CategoriesTreeView(object):
         # add the cell to the tvcolumn and allow it to expand
         self.tvcolumn.pack_start(self.cell, True)
 
-        ''' set the cell "text" attribute to column 0 - retrieve text
-            from that column in tree_store'''
+        """ set the cell "text" attribute to column 0 - retrieve text
+            from that column in tree_store"""
         #self.tvcolumn.add_attribute(self.cell, 'text', 0)
         self.tvcolumn.add_attribute(self.cell, 'markup', 0)
 
@@ -87,6 +89,15 @@ class CategoriesTreeView(object):
         self.cell.props.wrap_width = 200
         self.tree_view.connect_after("size-allocate", self.on_size_allocate, self.tvcolumn, self.cell)
 
+    def add_category(self, category):
+        """Add a new category name and sort all categories."""
+        if category:
+            self.last_category = category
+        if category is None or category in self.categories:
+            return
+        self.categories.append(category)
+        self.categories.sort(key=utils.sort_asc)
+
     def node_on_top_level(self, iter):
         if not type(iter) == gtk.TreeIter:
             # iter is a path -> convert to iter
@@ -102,19 +113,19 @@ class CategoriesTreeView(object):
         # Fetch the markup
         pango_markup = self.tree_store[path][0]
 
-        # Reset the renderer to use markup
+        # Tell the renderer NOT to use markup
         self.tvcolumn.clear_attributes(self.cell)
         self.tvcolumn.add_attribute(self.cell, 'markup', 0)
 
         # We want to show txt2tags markup and not pango markup
         editable.set_text(markup.convert_from_pango(pango_markup))
 
-    def edited_cb(self, cell, path, new_text, user_data):
-        '''
+    def edited_cb(self, cell, path, new_text, liststore):
+        """
         Called when text in a cell is changed
 
         new_text is txt2tags markup
-        '''
+        """
         if new_text == 'text' and self.node_on_top_level(path):
             self.statusbar.show_text('"text" is a reserved keyword', error=True)
             return
@@ -122,24 +133,11 @@ class CategoriesTreeView(object):
             self.statusbar.show_text(_('Empty entries are not allowed'), error=True)
             return
 
-        liststore = user_data
-        pango_markup = markup.convert_to_pango(new_text)
-        liststore[path][0] = pango_markup
+        liststore[path][0] = markup.convert_to_pango(new_text)
 
         # Category name changed
         if self.node_on_top_level(path):
-            if new_text not in self.categories:
-                self.categories.insert(0, new_text)
-
-        # Tag name changed
-        else:
-            iter = self.tree_store.get_iter(path)
-            iter_parent = self.tree_store.iter_parent(iter)
-            tags_iter = self._get_category_iter('Tags')
-
-            tags_node_is_parent = self.get_iter_value(iter_parent).capitalize() == 'Tags'
-            if tags_node_is_parent and self.node_on_top_level(iter_parent):
-                self.main_window.journal.save_old_day()
+            self.add_category(new_text)
 
         # Update cloud
         self.main_window.cloud.update()
@@ -149,22 +147,14 @@ class CategoriesTreeView(object):
             self.statusbar.show_text('"text" is a reserved keyword', error=True)
             return False
         if len(category) < 1:
-            self.statusbar.show_text(_('Empty category names are not allowed'), error=True)
+            self.statusbar.show_text(_('Empty tag names are not allowed'), error=True)
             return False
-
-        return True
-
-    def check_entry(self, text):
-        if len(text) < 1:
-            self.statusbar.show_text(_('Empty entries are not allowed'), error=True)
-            return False
-
         return True
 
     def add_element(self, parent, element_content):
-        '''
+        """
         Recursive Method for adding the content
-        '''
+        """
         # We want to order the entries ascendingly
         ascending = lambda (key, value): key.lower()
 
@@ -176,9 +166,8 @@ class CategoriesTreeView(object):
                 self.add_element(new_child, value)
 
     def set_day_content(self, day):
-        # We want to order the categories ascendingly, having Tags first
-        ascending = lambda x: '000' if x.lower() == 'tags' else x.lower()
-        sorted_keys = sorted(day.content.keys(), key=ascending)
+        # We want to order the categories ascendingly
+        sorted_keys = sorted(day.content.keys(), key=lambda x: x.lower())
 
         for key in sorted_keys:
             value = day.content[key]
@@ -209,11 +198,11 @@ class CategoriesTreeView(object):
             return content
 
     def empty(self, category_iter=None):
-        '''
+        """
         Tests whether a category has children
 
         If no category is given, test whether there are any categories
-        '''
+        """
         return self.tree_store.iter_n_children(category_iter) == 0
 
     def clear(self):
@@ -236,17 +225,18 @@ class CategoriesTreeView(object):
         return text
 
     def set_iter_value(self, iter, txt2tags_markup):
-        '''
-        text is txt2tags markup
-        '''
         pango_markup = markup.convert_to_pango(txt2tags_markup)
         self.tree_store.set_value(iter, 0, pango_markup)
 
-    def find_iter(self, category, entry):
+    def find_iter(self, category, entry=None):
         logging.debug('Looking for iter: "%s", "%s"' % (category, entry))
         category_iter = self._get_category_iter(category)
 
-        if not category_iter:
+        if category_iter:
+            # If we only search the category, return it.
+            if not entry:
+                return category_iter
+        else:
             # If the category was not found, return None
             return None
 
@@ -272,21 +262,20 @@ class CategoriesTreeView(object):
         return None
 
     def add_entry(self, category, entry, undoing=False):
-        if category not in self.categories and category is not None:
-            self.categories.insert(0, category)
+        self.add_category(category)
 
         category_iter = self._get_category_iter(category)
 
         entry_pango = markup.convert_to_pango(entry)
         category_pango = markup.convert_to_pango(category)
 
+        # If category exists add entry to existing category, else add new category
         if category_iter is None:
-            # If category does not exist add new category
             category_iter = self.tree_store.append(None, [category_pango])
-            entry_node = self.tree_store.append(category_iter, [entry_pango])
-        else:
-            # If category exists add entry to existing category
-            entry_node = self.tree_store.append(category_iter, [entry_pango])
+
+        # Only add entry if there is one
+        if entry_pango:
+            self.tree_store.append(category_iter, [entry_pango])
 
         if not undoing:
             undo_func = lambda: self.delete_node(self.find_iter(category, entry), undoing=True)
@@ -297,9 +286,9 @@ class CategoriesTreeView(object):
         self.tree_view.expand_all()
 
     def get_selected_node(self):
-        '''
+        """
         Returns selected node or None if none is selected
-        '''
+        """
         tree_selection = self.tree_view.get_selection()
         model, selected_iter = tree_selection.get_selected()
         return selected_iter
@@ -312,28 +301,25 @@ class CategoriesTreeView(object):
         # Save for undoing ------------------------------------
 
         # An entry is deleted
-        # We want to delete empty categories too
         if not self.node_on_top_level(iter):
-            deleting_entry = True
             category_iter = self.tree_store.iter_parent(iter)
             category = self.get_iter_value(category_iter)
             entries = [self.get_iter_value(iter)]
 
         # A category is deleted
         else:
-            deleting_entry = False
             category_iter = iter
             category = self.get_iter_value(category_iter)
-            entries = self._get_element_content(category_iter).keys()
+            content = self._get_element_content(category_iter)
+            if content:
+                entries = content.keys()
+            else:
+                entries = []
 
         # Delete ---------------------------------------------
 
         self.tree_store.remove(iter)
 
-        # Delete empty category
-        if deleting_entry and self.empty(category_iter):
-            self.tree_store.remove(category_iter)
-
         # ----------------------------------------------------
 
         if not undoing:
@@ -354,10 +340,6 @@ class CategoriesTreeView(object):
         self.main_window.cloud.update()
 
     def delete_selected_node(self):
-        '''
-        This method used to show a warning dialog. This has become obsolete
-        with the addition of undo functionality for the categories
-        '''
         selected_iter = self.get_selected_node()
         if selected_iter:
             self.delete_node(selected_iter)
@@ -406,22 +388,20 @@ class CategoriesTreeView(object):
             self.context_menu.popup(None, None, None, event.button, event.time)
 
     def _get_context_menu(self):
-        context_menu_xml = '''
+        context_menu_xml = """
         <ui>
         <popup action="ContextMenu">
             <menuitem action="ChangeEntry"/>
             <menuitem action="AddEntry"/>
             <menuitem action="Delete"/>
         </popup>
-        </ui>'''
+        </ui>"""
 
         uimanager = self.main_window.uimanager
 
         # Create an ActionGroup
         actiongroup = gtk.ActionGroup('ContextMenuActionGroup')
 
-        new_entry_dialog = self.main_window.new_entry_dialog
-
         # Create actions
         actiongroup.add_actions([
             ('ChangeEntry', gtk.STOCK_EDIT,
@@ -475,13 +455,13 @@ class CategoriesTreeView(object):
         self.delete_selected_node()
 
     def on_size_allocate(self, treeview, allocation, column, cell):
-        '''
+        """
         Code from pychess project
         (http://code.google.com/p/pychess/source/browse/trunk/lib/pychess/
         System/uistuff.py?r=1025#62)
 
         Allows dynamic line wrapping in a treeview
-        '''
+        """
         other_columns = (c for c in treeview.get_columns() if c != column)
         new_width = allocation.width - sum(c.get_width() for c in other_columns)
         new_width -= treeview.style_get_property("horizontal-separator") * 2
