@@ -25,13 +25,6 @@ import gtk
 import gobject
 
 
-def get_button_width(button, label):
-    button.set_label(label)
-    while gtk.events_pending():
-        gtk.main_iteration()
-    return button.allocation.width
-
-
 class ActionButton(gtk.Button):
     def __init__(self, text, action):
         gtk.Button.__init__(self, text)
@@ -49,8 +42,6 @@ class CustomComboBoxEntry(object):
     def __init__(self, combo_box):
         self.combo_box = combo_box
 
-        #self.liststore = self.combo_box.get_model()
-        #if self.liststore is None:
         self.liststore = gtk.ListStore(gobject.TYPE_STRING)
         self.combo_box.set_model(self.liststore)
         #self.combo_box.set_wrap_width(5)
@@ -69,13 +60,10 @@ class CustomComboBoxEntry(object):
 
     def set_entries(self, value_list):
         self.clear()
+        self.combo_box.set_model(None)
         for entry in value_list:
             self.add_entry(entry)
-
-        if len(value_list) > 0:
-            self.combo_box.set_active(0)
-            self.set_active_text(value_list[0])
-            self.combo_box.queue_draw()
+        self.combo_box.set_model(self.liststore)
 
     def get_active_text(self):
         return self.entry.get_text().decode('utf-8')
@@ -84,9 +72,11 @@ class CustomComboBoxEntry(object):
         return self.entry.set_text(text)
 
     def clear(self):
+        self.combo_box.set_model(None)
         if self.liststore:
             self.liststore.clear()
         self.set_active_text('')
+        self.combo_box.set_model(self.liststore)
 
     def connect(self, *args, **kargs):
         self.combo_box.connect(*args, **kargs)
@@ -96,12 +86,16 @@ class CustomComboBoxEntry(object):
 
 
 class CustomListView(gtk.TreeView):
-    def __init__(self):
+    def __init__(self, columns):
+        """
+        *columns* must be a list of (header, type) pairs e.g. [('title', str)].
+        """
         gtk.TreeView.__init__(self)
-        # create a TreeStore with two string columns to use as the model
-        self.set_model(gtk.ListStore(str, str))
+        headers, types = zip(*columns)
+        # create a TreeStore with columns to use as the model
+        self.set_model(gtk.ListStore(*types))
 
-        columns = [gtk.TreeViewColumn('1'), gtk.TreeViewColumn('2')]
+        columns = [gtk.TreeViewColumn(header) for header in headers]
 
         # add tvcolumns to tree_view
         for index, column in enumerate(columns):
@@ -123,46 +117,11 @@ class CustomListView(gtk.TreeView):
         self.set_search_column(1)
 
 
-class EntryDialog(gtk.MessageDialog):
-    # base this on a message dialog
-    def __init__(self, title, value_name, subtitle=''):
-        gtk.MessageDialog.__init__(self, None,
-                                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-                                gtk.MESSAGE_QUESTION,
-                                gtk.BUTTONS_OK,
-                                None)
-        self.set_markup(title)
-
-        # create the text input field
-        self.entry = gtk.Entry()
-
-        # allow the user to press enter to do ok
-        def response_to_dialog(entry, response):
-            self.response(response)
-        self.entry.connect("activate", response_to_dialog, gtk.RESPONSE_OK)
-
-        # create a horizontal box to pack the entry and a label
-        hbox = gtk.HBox()
-        hbox.pack_start(gtk.Label(value_name), False, 5, 5)
-        hbox.pack_end(self.entry)
-
-        if subtitle:
-            # some secondary text
-            self.format_secondary_markup(subtitle)
-
-        # add it and show it
-        self.vbox.pack_end(hbox, True, True, 0)
-        self.show_all()
-
-    def get_value(self):
-        return self.entry.get_text()
-
-
 class Calendar(gtk.Calendar):
     def __init__(self, week_numbers=False):
         gtk.Calendar.__init__(self)
         if week_numbers:
-            calendar.set_property('show-week-numbers', True)
+            self.set_property('show-week-numbers', True)
 
     def set_date(self, date):
         '''
@@ -178,12 +137,54 @@ class Calendar(gtk.Calendar):
         # Select the day after the month and year have been set
         self.select_day(date.day)
 
-
     def get_date(self):
         year, month, day = gtk.Calendar.get_date(self)
         return datetime.date(year, month+1, day)
 
 
+# gtk.InfoBar is available in gtk+ >= 2.22
+if hasattr(gtk, 'InfoBar'):
+    class Info(gtk.InfoBar):
+        icons = {gtk.MESSAGE_ERROR: gtk.STOCK_DIALOG_ERROR}
+        def __init__(self, *args, **kwargs):
+            gtk.InfoBar.__init__(self, *args, **kwargs)
+            self.title_label = gtk.Label()
+            self.msg_label = gtk.Label()
+            self.title_label.set_alignment(0., 0.5)
+            self.msg_label.set_alignment(0., 0.5)
+
+            vbox = gtk.VBox(spacing=5)
+            vbox.pack_start(self.title_label, False, False)
+            vbox.pack_start(self.msg_label, False, False)
+
+            self.image = gtk.Image()
+
+            content = self.get_content_area()
+            content.pack_start(self.image, False)
+            content.pack_start(vbox, False)
+
+            self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+            self.connect('close', lambda x: self.hide())
+            self.connect('response', self.on_response)
+
+        def on_response(self, infobar, response_id):
+            if response_id == gtk.RESPONSE_CLOSE:
+                self.hide()
+
+        def show_message(self, title, msg, msg_type):
+            if not title:
+                title = msg
+                msg = ''
+            self.title_label.set_markup('<b>%s</b>' % title)
+            self.msg_label.set_markup(msg)
+            self.set_message_type(msg_type)
+            self.image.set_from_stock(self.icons.get(msg_type, gtk.STOCK_DIALOG_INFO),
+                                      gtk.ICON_SIZE_DIALOG)
+            self.show_all()
+else:
+    Info = None
+
+
 # ------------------------- Assistant Pages ------------------------------------
 
 class AssistantPage(gtk.VBox):
@@ -295,7 +296,7 @@ class PathChooserPage(AssistantPage):
         elif self.path_type == 'NEWFILE':
             self.chooser.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
         else:
-            logging.error('Wrong path_type "%s"' % path_type)
+            logging.error('Wrong path_type "%s"' % self.path_type)
 
         if self.path_type in ['FILE', 'NEWFILE'] and extension:
             filter = gtk.FileFilter()
@@ -303,39 +304,26 @@ class PathChooserPage(AssistantPage):
             filter.add_pattern('*.' + extension)
             self.chooser.add_filter(filter)
 
-        #path = self.last_path or path
+        if self.last_path and os.path.exists(self.last_path):
+            path = self.last_path
 
         if os.path.isdir(path):
             self.chooser.set_current_folder(path)
         else:
-            if os.path.exists(path):
-                # Method is for existing files
-                self.chooser.set_filename(path)
-            else:
-                self.chooser.set_current_folder(os.path.dirname(path))
-                self.chooser.set_current_name(os.path.basename(path))
+            dirname, basename = os.path.split(path)
+            filename, old_ext = os.path.splitext(basename)
+            self.chooser.set_current_folder(dirname)
+            self.chooser.set_current_name(filename + '.' + extension)
 
 
     def get_selected_path(self):
-        self.last_path = self.chooser.get_filename()
+        self.last_path = self.chooser.get_filename().decode('utf-8')
         return self.last_path
 
 
     def on_path_changed(self, widget):
-        # TODO: Try to make this smarter
         return
 
-        correct = False
-        path = self.chooser.get_filename()
-        if path is None:
-            correct = False
-        elif self.path_type == 'DIR':
-            correct = os.path.isdir(path)
-        elif self.path_type == 'FILE':
-            correct = os.path.isfile(path)
-        elif self.path_type == 'NEWFILE':
-            correct = os.path.isfile(path)
-
 
 
 class Assistant(gtk.Assistant):
