@@ -25,6 +25,7 @@ from rednotebook.util import utils
 from rednotebook import info
 from rednotebook.util import filesystem
 from rednotebook.util import markup
+from rednotebook import storage
 #from rednotebook.gui.imports import ImportAssistant
 
 
@@ -102,15 +103,15 @@ class MainMenuBar(object):
                 None, _('Save journal at a new location. The old journal files will also be saved'),
                 self.on_save_as_menu_item_activate),
             ### Translators: Verb
-            ('Import', gtk.STOCK_ADD, _('Import'),
+            ('Import', gtk.STOCK_ADD, _('_Import'),
                 None, _('Open the import assistant'), self.on_import_menu_item_activate),
             ### Translators: Verb
             ('Export', gtk.STOCK_CONVERT, _('Export'),
-                None, _('Open the export assistant'), self.on_export_menu_item_activate),
+                '<Ctrl>e', _('Open the export assistant'), self.on_export_menu_item_activate),
             ### Translators: Verb
-            ('Backup', gtk.STOCK_HARDDISK, _('Backup'),
+            ('Backup', gtk.STOCK_HARDDISK, _('_Backup'),
                 None, _('Save all the data in a zip archive'), self.on_backup_activate),
-            ('Statistics', None, _('Statistics'),
+            ('Statistics', None, _('S_tatistics'),
                 None, _('Show some statistics about the journal'), self.on_statistics_menu_item_activate),
             ('Quit', gtk.STOCK_QUIT, None,
                 None, _('Shutdown RedNotebook. It will not be sent to the tray.'),
@@ -159,42 +160,57 @@ class MainMenuBar(object):
         self.menubar = self.uimanager.get_widget('/MainMenuBar')
         return self.menubar
 
-    def set_tooltips(self):
-        groups = self.uimanager.get_action_groups()
-        for group in groups:
-            actions = group.list_actions()
-            for action in actions:
-                widgets = action.get_proxies()
-                tooltip = action.get_property('tooltip')
-                if tooltip:
-                    for widget in widgets:
-                        widget.set_tooltip_markup(tooltip)
+    def check_journal_dir(self, action, new_dir):
+        if not new_dir:
+            return False
+        title = _('Wrong directory')
+        # Check if dir exists and is no forbidden path (e.g. $HOME dir).
+        if not self.journal.dirs.is_valid_journal_path(new_dir):
+            self.journal.show_message(_('You cannot use this directory for your journal:') +
+                                      ' %s' % new_dir, title=title, error=True)
+            return False
+        if action in ['new', 'saveas'] and os.listdir(new_dir):
+            self.journal.show_message(_('Please select an empty directory.'),
+                                      title=title, error=True)
+            return False
+        elif action in ['open'] and not list(storage.get_journal_files(new_dir)):
+            self.journal.show_message(_('This directory contains no journal files:') +
+                                      ' ' + new_dir, title=title, error=True)
+            return False
+        return True
+
+    def select_journal(self, action, title, message):
+        new_dir = self.main_window.get_new_journal_dir(title, message)
+        if not self.check_journal_dir(action, new_dir):
+            return
+
+        if action == 'saveas':
+            self.journal.dirs.data_dir = new_dir
+            self.journal.save_to_disk(saveas=True)
+        self.journal.open_journal(new_dir)
 
     def on_new_journal_button_activate(self, widget):
-        self.main_window.show_dir_chooser('new')
+        msg = ('%s\n%s' % (_('Journals are saved in a directory, not in a single file.'),
+                           _('The directory name will be the title of the new journal.')))
+        self.select_journal('new', _('Select an empty folder for your new journal'), msg)
 
     def on_open_journal_button_activate(self, widget):
-        self.main_window.show_dir_chooser('open')
+        self.select_journal('open', _('Select an existing journal directory'),
+                            _("The directory should contain your journal's data files"))
 
     def on_save_button_clicked(self, widget):
         self.journal.save_to_disk()
 
     def on_save_as_menu_item_activate(self, widget):
         self.journal.save_to_disk()
-
-        self.main_window.show_dir_chooser('saveas')
+        self.select_journal('saveas', _('Select an empty folder for the new location of your journal'),
+                            _('The directory name will be the new title of the journal'))
 
     def on_edit_menu_activate(self, widget):
-        '''
-        Only set the menu items for undo and redo sensitive if the actions
-        can really be performed
-
-        Probably useless since the buttons are set in undo.py
-        '''
-        can_undo = self.main_window.undo_redo_manager.can_undo()
-        self.main_window.uimanager.get_widget('/MainMenuBar/Edit/Undo').set_sensitive(can_undo)
-        can_redo = self.main_window.undo_redo_manager.can_redo()
-        self.main_window.uimanager.get_widget('/MainMenuBar/Edit/Redo').set_sensitive(can_redo)
+        """Only set the menu items sensitive if the actions can be performed."""
+        edit_mode = not self.main_window.preview_mode
+        for action in ['Cut', 'Paste']:
+            self.main_window.uimanager.get_widget('/MainMenuBar/Edit/%s' % action).set_sensitive(edit_mode)
 
     def on_undo(self, widget):
         self.main_window.undo_redo_manager.undo()
@@ -202,18 +218,20 @@ class MainMenuBar(object):
     def on_redo(self, widget):
         self.main_window.undo_redo_manager.redo()
 
+    def _get_active_editor_widget(self):
+        if self.main_window.preview_mode:
+            return self.main_window.html_editor.webview
+        else:
+            return self.main_window.day_text_field.day_text_view
+
     def on_copy_menu_item_activate(self, widget):
-        self.main_window.day_text_field.day_text_view.emit('copy_clipboard')
+        self._get_active_editor_widget().emit('copy_clipboard')
 
     def on_paste_menu_item_activate(self, widget):
-        self.main_window.day_text_field.day_text_view.emit('paste_clipboard')
+        self._get_active_editor_widget().emit('paste_clipboard')
 
     def on_cut_menu_item_activate(self, widget):
-#       event = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
-#       event.keyval = ord("X")
-#       event.state = gtk.gdk.CONTROL_MASK
-#       self.main_frame.emit("key_press_event",event)
-        self.main_window.day_text_field.day_text_view.emit('cut_clipboard')
+        self._get_active_editor_widget().emit('cut_clipboard')
 
     def on_fullscreen_menuitem_activate(self, widget):
         self.main_window.toggle_fullscreen()
@@ -263,14 +281,15 @@ class MainMenuBar(object):
         self.info_dialog.set_transient_for(self.main_window.main_frame)
         self.info_dialog.set_name('RedNotebook')
         self.info_dialog.set_version(info.version)
-        self.info_dialog.set_copyright('Copyright (c) 2008-2011 Jendrik Seipp')
+        self.info_dialog.set_copyright('Copyright (c) 2008-2012 Jendrik Seipp')
         self.info_dialog.set_comments(_('A Desktop Journal'))
         gtk.about_dialog_set_url_hook(lambda dialog, url: webbrowser.open(url))
         self.info_dialog.set_website(info.url)
         self.info_dialog.set_website_label(info.url)
         self.info_dialog.set_authors(info.developers)
-        self.info_dialog.set_logo(gtk.gdk.pixbuf_new_from_file(\
-                    os.path.join(filesystem.image_dir,'rednotebook-icon/rn-128.png')))
+        # TODO: Use svg again once it's fixed under Windows.
+        self.info_dialog.set_logo(gtk.gdk.pixbuf_new_from_file(
+                os.path.join(filesystem.image_dir, 'rednotebook-icon', 'rn-192.png')))
         self.info_dialog.set_license(info.license_text)
         self.info_dialog.run()
         self.info_dialog.hide()
