@@ -23,11 +23,9 @@ import logging
 import platform
 
 import gtk
-import gobject
 
 from rednotebook.gui.customwidgets import UrlButton, CustomComboBoxEntry
 from rednotebook.gui.customwidgets import ActionButton
-from rednotebook.gui import browser
 from rednotebook.util import filesystem, utils, dates
 from rednotebook import info
 
@@ -51,19 +49,19 @@ class Option(gtk.HBox):
         raise NotImplementedError
 
     def get_string_value(self):
-        return unicode(self.get_value()).strip()
+        return unicode(self.get_value())
 
 
 class TickOption(Option):
-    def __init__(self, text, name, default_value=None, tooltip=''):
+    def __init__(self, text, name, value=None, default_value=0, tooltip=''):
         Option.__init__(self, '', name, tooltip=tooltip)
 
         self.check_button = gtk.CheckButton(text)
 
-        if default_value is None:
-            self.check_button.set_active(Option.config.read(name, 0) == 1)
+        if value is None:
+            self.check_button.set_active(Option.config.read(name, default_value) == 1)
         else:
-            self.check_button.set_active(default_value)
+            self.check_button.set_active(value)
 
         self.pack_start(self.check_button, False)
 
@@ -84,7 +82,7 @@ class AutostartOption(TickOption):
         self.autostart_file = os.path.join(autostart_dir, 'rednotebook.desktop')
         autostart_file_exists = os.path.exists(self.autostart_file)
         TickOption.__init__(self, _('Load RedNotebook at startup'), None,
-                        default_value=autostart_file_exists)
+                            value=autostart_file_exists)
 
     def get_value(self):
         return self.check_button.get_active()
@@ -289,7 +287,12 @@ class OptionsManager(object):
             logging.debug('Running on Linux. Is installed. Adding autostart option')
             self.options.insert(0, AutostartOption())
 
-        self.options.append(TickOption(_('Close to system tray'), 'closeToTray',
+        # Most modern Linux distributions do not have a systray anymore.
+        # If this option is activated on a system without a systray, the
+        # application keeps on running in the background after it has been
+        # closed. The option can still be activated in the configuration file.
+        if sys.platform.startswith('win'):
+            self.options.append(TickOption(_('Close to system tray'), 'closeToTray',
                 tooltip=_('Closing the window will send RedNotebook to the tray')))
 
         able_to_spell_check = self.main_window.day_text_field.can_spell_check()
@@ -302,9 +305,13 @@ class OptionsManager(object):
             self.options.append(spell_check_option)
         spell_check_option.set_sensitive(able_to_spell_check)
 
-        # Check for new version
+        # Automatic switching between preview and edit mode.
+        self.options.append(TickOption(_('Switch between edit and preview mode automatically'),
+                                       'autoSwitchMode', default_value=1))
 
+        # Check for new version
         check_version_option = TickOption(_('Check for new version at startup'), 'checkForNewVersion')
+
         def check_version_action(widget):
             utils.check_new_version(self.main_window.journal, info.version)
             # Apply changes from dialog to options window
@@ -318,10 +325,10 @@ class OptionsManager(object):
         self.options.extend([
                 FontSizeOption(_('Font Size'), 'mainFontSize'),
                 DateFormatOption(_('Date/Time format'), 'dateTimeString'),
-                CsvTextOption(_('Exclude from clouds'), 'cloudIgnoreList',
-                                tooltip=_('Do not show those comma separated words in any cloud')),
-                CsvTextOption(_('Allow small words in clouds'), 'cloudIncludeList',
-                                tooltip=_('Allow those words with 4 letters or less in the text cloud')),
+                CsvTextOption(_('Exclude from cloud'), 'cloudIgnoreList',
+                                tooltip=_('Do not show these comma separated words in the word cloud')),
+                CsvTextOption(_('Include small words in cloud'), 'cloudIncludeList',
+                                tooltip=_('Allow these words with 4 letters or less')),
                 ])
 
 
