@@ -17,10 +17,10 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-'''
+"""
 This module takes the ideas and some code from the highlighting module
 PyGTKCodeBuffer by Hannes Matuschek (http://code.google.com/p/pygtkcodebuffer/).
-'''
+"""
 
 import gtk
 import sys
@@ -33,9 +33,8 @@ if __name__ == '__main__':
     sys.path.insert(0, os.path.abspath("./../../"))
     logging.getLogger('').setLevel(logging.DEBUG)
 
-#from rednotebook.external import gtkcodebuffer
+from rednotebook.data import HASHTAG
 from rednotebook.external import txt2tags
-#from rednotebook.gui.richtext import HtmlEditor
 from rednotebook.gui.browser import HtmlView
 from rednotebook.util import markup
 
@@ -78,16 +77,16 @@ class TagGroup(list):
 
 
 class Pattern(object):
-    '''
+    """
     A pattern object allows a regex-pattern to have
     subgroups with different formatting
-    '''
+    """
     def __init__(self, pattern, group_tag_pairs, regex=None, flags="",
-                        overlap=False, name='unnamed'):
-        self.overlap = overlap
+                 name='unnamed'):
         self.name = name
 
         # assemble re-flag
+        # TODO: Is this needed?
         flags += "ML"
         flag = 0
 
@@ -130,13 +129,11 @@ class Pattern(object):
 
 
 class MarkupDefinition(object):
-
     def __init__(self, rules):
         self.rules = rules
         self.highlight_rule = None
 
     def __call__(self, buf, start, end):
-
         txt = buf.get_slice(start, end)
 
         tag_groups = []
@@ -160,7 +157,6 @@ class MarkupDefinition(object):
 
 
 class MarkupBuffer(gtk.TextBuffer):
-
     def __init__(self, table=None, lang=None, styles={}):
         gtk.TextBuffer.__init__(self, table)
 
@@ -185,14 +181,14 @@ class MarkupBuffer(gtk.TextBuffer):
     def set_search_text(self, text):
         if not text:
             self._lang_def.highlight_rule = None
-        self._lang_def.highlight_rule = Pattern(r"(%s)" % text,  [(1, 'highlight')],
-                                name='highlight', flags='I', overlap=True)
+        self._lang_def.highlight_rule = Pattern(r"(%s)" % re.escape(text),
+                            [(1, 'highlight')], name='highlight', flags='I')
         self.update_syntax(self.get_start_iter(), self.get_end_iter())
 
     def get_slice(self, start, end):
-        '''
+        """
         We have to search for the regexes in utf-8 text
-        '''
+        """
         slice_text = gtk.TextBuffer.get_slice(self, start, end)
         slice_text = slice_text.decode('utf-8')
         return slice_text
@@ -210,9 +206,9 @@ class MarkupBuffer(gtk.TextBuffer):
         self.update_syntax(start, start)
 
     def remove_all_syntax_tags(self, start, end):
-        '''
+        """
         Do not remove the gtkspell highlighting
-        '''
+        """
         for style in self.styles:
             self.remove_tag_by_name(style, start, end)
 
@@ -222,14 +218,11 @@ class MarkupBuffer(gtk.TextBuffer):
             self.apply_tag_by_name(tagname, mstart, mend)
 
     def update_syntax(self, start, end):
-        """ More or less internal used method to update the
-            syntax-highlighting. """
-        '''
-        Use two categories of rules: one-line and multiline
-
-        Before running multiline rules: Check if e.g. - is present in changed string
-        '''
+        """Use two categories of rules: one-line and multiline
 
+        Before running multiline rules: Check if e.g. - is present in changed
+        string.
+        """
         # Just update from the start of the first edited line
         # to the end of the last edited line, because we can
         # guarantee that there's no multiline rule
@@ -280,6 +273,10 @@ styles = {  'bold':             {'weight': pango.WEIGHT_BOLD},
             'link':             {'foreground': 'blue',
                                 'underline': pango.UNDERLINE_SINGLE,},
             'highlight':        {'background': 'yellow'},
+            'quote':            {'background': 'gray'},
+            'tablehead':        {'background': markup.TABLE_HEAD_BG},
+            'tablerow':         {'background': '#eee'},
+            'formula':          {'style': pango.STYLE_ITALIC, 'family': 'serif'}
             }
 def add_header_styles():
     sizes = [
@@ -308,8 +305,7 @@ def get_pattern(char, style):
     # Either one char, or two chars with (maybe empty) content
     # between them
     # In both cases no whitespaces between chars and markup
-    #regex = r'(%s)(\S.*\S)(%s)' % ((markup_symbols, ) * 2)
-    regex = r'(%s%s)(\S|.*?\S%s*)(%s%s)' % ((char, ) * 5)
+    regex = r'(%s%s)(\S|\S.*?\S%s*)(%s%s)' % ((char, ) * 5)
     group_style_pairs = [(1, 'gray'), (2, style), (3, 'gray')]
     return Pattern(regex, group_style_pairs, name=style)
 
@@ -332,7 +328,7 @@ for level in range(1, 6):
     numtitle = Pattern(numtitle_pattern, title_style + [(2, style_name)])
     title_patterns += [title, numtitle]
 
-linebreak = Pattern(r'(\\\\)', [(1, 'gray')])
+linebreak = Pattern(r'(%s)' % markup.REGEX_LINEBREAK, [(1, 'gray')])
 
 # pic [""/home/user/Desktop/RedNotebook pic"".png]
 # \w = [a-zA-Z0-9_]
@@ -340,7 +336,7 @@ linebreak = Pattern(r'(\\\\)', [(1, 'gray')])
 # filename = One char or two chars with possibly whitespace in the middle
 #filename = r'\S[\w\s_,.+%$#@!?+~/-:-\(\)]*\S|\S'
 filename = r'\S.*?\S|\S'
-ext = r'png|jpe?g|gif|eps|bmp'
+ext = r'(png|jpe?g|gif|eps|bmp)'
 pic = Pattern(r'(\["")(%s)("")(\.%s)(\?\d+)?(\])' % (filename, ext),
         [(1, 'gray'), (2, 'green'), (3, 'gray'), (4, 'green'), (5, 'gray'), (6, 'gray')], flags='I')
 
@@ -350,12 +346,21 @@ named_link = Pattern(r'(\[)(.*?)\s("")(\S.*?\S)(""\])',
         [(1, 'gray'), (2, 'link'), (3, 'gray'), (4, 'gray'), (5, 'gray')], flags='LI')
 
 # link http://heise.de
-# Use txt2tags link guessing mechanism
-link = Pattern('OVERWRITE', [(0, 'link')], regex=bank['link'], name='link')
+# Use txt2tags link guessing mechanism by setting regex explicitly
+link = Pattern('UseRegex', [(0, 'link')], regex=bank['link'], name='link')
 
 # We do not support multiline regexes
 #blockverbatim = Pattern(r'^(```)\s*$\n(.*)$\n(```)\s*$', [(1, 'gray'), (2, 'verbatim'), (3, 'gray')])
 
+quote = Pattern(r'^\t+(.*)$', [(1, 'quote')])
+
+table_head = Pattern(r'^ *(\|\| .*)', [(1, 'tablehead')])
+table_row = Pattern(r'^ *(\| .*)', [(1, 'tablerow')])
+
+formula = Pattern(r'(\\\(|\\\[|\$\$)(.+?)(\\\)|\\\]|\$\$)', [(1, 'gray'), (2, 'formula'), (3, 'gray')])
+
+hashtag = Pattern('UseRegex', [(2, 'red'), (3, 'red')], regex=HASHTAG)
+
 
 patterns = [
         get_pattern('\*', 'bold'),
@@ -373,6 +378,11 @@ patterns = [
         pic,
         named_link,
         link,
+        quote,
+        table_head,
+        table_row,
+        formula,
+        hashtag,
         ] + title_patterns
 
 
@@ -445,8 +455,7 @@ www.heise.de, andy@web.de
 
     def change_text(widget):
         html = markup.convert(widget.get_text(widget.get_start_iter(),
-                              widget.get_end_iter()), 'xhtml',
-                              append_whitespace=True)
+                              widget.get_end_iter()), 'xhtml')
 
         html_editor.load_html(html)
         html_editor.highlight(search_text)
