@@ -17,16 +17,15 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import sys
 import datetime
 import os
-import operator
-import collections
 import time
+import itertools
 import logging
-from optparse import OptionParser, OptionValueError
+import locale
+from optparse import OptionParser
+from collections import defaultdict
 
 
 # Use basic stdout logging before we can initialize logging correctly
@@ -41,79 +40,36 @@ if hasattr(sys, "frozen"):
 else:
     from util import filesystem
 
+# Add base directory to sys.path
 base_dir = os.path.abspath(os.path.join(filesystem.app_dir, '../'))
-if base_dir not in sys.path:
-    # Adding BaseDir to sys.path
-    sys.path.insert(0, base_dir)
-
-#from rednotebook.util import filesystem # creates a copy of filesystem module
-#import rednotebook.util.filesystem      # imports the original filesystem module
+sys.path.insert(0, base_dir)
 
 
 ## ---------------------- Enable i18n -------------------------------
 
+from rednotebook.external import elibintl
+
 # We need to translate 3 different types of strings:
 # * sourcecode strings
 # * gtkbuilder strings
 # * gtk stock names
 
-# set the locale for all categories to the user’s default setting
-# (typically specified in the LANG environment variable)
-import locale
-lang = os.environ.get('LANG', None)
-logging.info('LANG: %s' % lang)
-default_locale = locale.getdefaultlocale()[0]
-logging.info('Default locale: %s' % default_locale)
-try:
-    locale.setlocale(locale.LC_ALL, '')
-    logging.info('Set default locale: "%s"' % default_locale)
-except locale.Error, err:
-    # unsupported locale setting
-    logging.error('Locale "%s" could not be set: "%s"' % (default_locale, err))
-    logging.error('Probably you have to install the appropriate language packs')
+LOCALE_PATH = filesystem.get_utf8_path(filesystem.locale_dir)
 
-# If the default locale could be determined and the LANG env variable
-# has not been set externally, set LANG to the default locale
-# This is necessary only for windows where program strings are not
-# shown in the system language, but in English
-if default_locale and not lang:
-    logging.info('Setting LANG to %s' % default_locale)
-    # sourcecode strings
-    os.environ['LANG'] = default_locale
-
-LOCALE_PATH = os.path.join(filesystem.app_dir, 'i18n')
-
-# the name of the gettext domain. because we have our translation files
-# not in a global folder this doesn't really matter, setting it to the
-# application name is a good idea tough.
+# the name of the gettext domain.
 GETTEXT_DOMAIN = 'rednotebook'
 
-# set up the gettext system
-import gettext
-
-# Adding locale to the list of modules translates gtkbuilder strings
-modules = [#gettext,
-            locale]
-
-# Sometimes this doesn't work though,
-# so we try to call gtk.glade's function as well if glade is present
+# Register _() as a global translation function and set up the translation
 try:
-    import gtk.glade
-    modules.append(gtk.glade)
-    logging.info('Module glade found')
-except ImportError, err:
-    logging.info('Module glade not found: %s' % err)
-
-for module in modules:
-    try:
-        # locale.bintextdomain and locale textdomain not available on win
-        module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
-        module.textdomain(GETTEXT_DOMAIN)
-    except AttributeError, err:
-        logging.info(err)
-
-# register the gettext function for the whole interpreter as "_"
-gettext.install(GETTEXT_DOMAIN, LOCALE_PATH, unicode=1)
+    elibintl.install(GETTEXT_DOMAIN, LOCALE_PATH)
+except locale.Error, err:
+    # unsupported locale setting
+    logging.error('Locale could not be set: "%s"' %  err)
+    logging.error('Probably you have to install the appropriate language packs')
+    # Make the _() function available even if gettext is not working.
+    import __builtin__
+    if not hasattr(__builtin__, '_'):
+        __builtin__.__dict__['_'] = lambda s: s
 
 ## ------------------- end Enable i18n -------------------------------
 
@@ -121,7 +77,7 @@ gettext.install(GETTEXT_DOMAIN, LOCALE_PATH, unicode=1)
 from rednotebook.util import utils
 from rednotebook import info
 from rednotebook import configuration
-
+from rednotebook import data
 
 
 def parse_options():
@@ -150,20 +106,15 @@ options, args = parse_options()
 ## ---------------------- Enable logging -------------------------------
 
 def setup_logging(log_file):
-    #logging_levels = {'debug': logging.DEBUG,
-    #               'info': logging.INFO,
-    #               'warning': logging.WARNING,
-    #               'error': logging.ERROR,
-    #               'critical': logging.CRITICAL}
-
-    # File logging
-    if sys.platform == 'win32' and hasattr(sys, "frozen"):
-        utils.redirect_output_to_file(log_file)
-
     file_logging_stream = open(log_file, 'w')
 
-    # We want to have the error messages in the logfile
-    sys.stderr = utils.StreamDuplicator(sys.__stderr__, [file_logging_stream])
+    # We want to have the error messages in the logfile. In the frozen
+    # version we cannot log to sys.stderr because it's broken on
+    # windows. Stdout is redirected into a blackhole by py2exe.
+    stderr_streams = [file_logging_stream]
+    if not hasattr(sys, 'frozen'):
+        stderr_streams.append(sys.__stderr__)
+    sys.stderr = utils.StreamDuplicator(stderr_streams)
 
     root_logger = logging.getLogger('')
     root_logger.setLevel(logging.DEBUG)
@@ -275,7 +226,9 @@ class Journal:
         self.title = ''
 
         # show instructions at first start
-        logging.info('First Start: %s' % self.dirs.is_first_start)
+        self.is_first_start = self.config.read('firstStart', 1)
+        self.config['firstStart'] = 0
+        logging.info('First Start: %s' % bool(self.is_first_start))
 
         logging.info('RedNotebook version: %s' % info.version)
         logging.info(filesystem.get_platform_info())
@@ -288,12 +241,20 @@ class Journal:
         self.frame = None
         self.frame = MainWindow(self)
 
-        self.open_journal(self.get_journal_path())
+        journal_path = self.get_journal_path()
+        if not self.dirs.is_valid_journal_path(journal_path):
+            logging.error('Invalid directory: %s. Using default journal.' % journal_path)
+            self.show_message(_('You cannot use this directory for your journal:') +
+                              ' %s' % journal_path + '. ' + _('Opening default journal.'),
+                              error=True)
+            journal_path = self.dirs.default_data_dir
+        self.open_journal(journal_path)
 
         self.archiver = backup.Archiver(self)
+        #self.archiver.check_last_backup_date()
 
         # Check for a new version
-        if self.config.read('checkForNewVersion', default=0) == 1:
+        if self.config.read('checkForNewVersion', 0) == 1:
             utils.check_new_version(self, info.version, startup=True)
 
         # Automatically save the content after a period of time
@@ -336,16 +297,11 @@ class Journal:
         sys.exit(1)
 
 
-    def backup_contents(self, backup_file):
-        self.save_to_disk()
-
-        if backup_file:
-            self.archiver.backup(backup_file)
-
-
     def exit(self):
         self.frame.add_values_to_config()
 
+        self.config['running'] = 0
+
         # Make it possible to stop the program from exiting
         # e.g. if the journal could not be saved
         self.is_allowed_to_exit = True
@@ -353,43 +309,31 @@ class Journal:
 
         if self.is_allowed_to_exit:
             logging.info('Goodbye!')
+            # Informs the logging system to perform an orderly shutdown by
+            # flushing and closing all handlers.
+            logging.shutdown()
             gtk.main_quit()
 
 
     def save_to_disk(self, exit_imminent=False, changing_journal=False, saveas=False):
-        #logging.info('Trying to save the journal')
-
         self.save_old_day()
 
         try:
             filesystem.make_directory(self.dirs.data_dir)
-        except OSError, err:
+        except OSError:
             self.frame.show_save_error_dialog(exit_imminent)
             return True
 
-        if not os.path.exists(self.dirs.data_dir):
-            logging.error('Save path does not exist')
-            self.frame.show_save_error_dialog(exit_imminent)
-            return True
-
-
         something_saved = storage.save_months_to_disk(self.months,
             self.dirs.data_dir, self.frame, exit_imminent, changing_journal, saveas)
 
         if something_saved:
             self.show_message(_('The content has been saved to %s') % self.dirs.data_dir, error=False)
-            logging.info('The content has been saved to %s' % self.dirs.data_dir)
+            logging.info('The content has been saved to %r' % self.dirs.data_dir)
         else:
             self.show_message(_('Nothing to save'), error=False)
-            #logging.info('Nothing to save')
 
-        if self.config.changed():
-            try:
-                filesystem.make_directory(self.dirs.journal_user_dir)
-                self.config.save_to_disk()
-            except IOError, err:
-                self.show_message(_('Configuration could not be saved. Please check your permissions'))
-                logging.error('Configuration could not be saved. Please check your permissions')
+        self.config.save_to_disk()
 
         if not (exit_imminent or changing_journal) and something_saved:
             # Update cloud
@@ -399,71 +343,41 @@ class Journal:
         return True
 
 
-    def open_journal(self, data_dir, load_files=True):
+    def open_journal(self, data_dir):
+        if not os.path.exists(data_dir):
+            logging.warning('The dir %s does not exist. Select a different dir.'
+                            % data_dir)
+            return
 
         if self.months:
             self.save_to_disk(changing_journal=True)
 
-        # Password Protection
-        #password = self.config.read('password', '')
-
-        logging.info('Opening journal at %s' % data_dir)
-
-        if not os.path.exists(data_dir):
-            logging.warning('The data dir %s does not exist. Select a different dir.'
-                        % data_dir)
-
-            self.frame.show_dir_chooser('open', dir_not_found=True)
-            return
-
-        data_dir_empty = not os.listdir(data_dir)
-
-        if not load_files and not data_dir_empty:
-            msg_part1 = _('The selected folder is not empty.')
-            msg_part2 = _('To prevent you from overwriting data, the folder content has been imported into the new journal.')
-            self.show_message('%s %s' % (msg_part1, msg_part2), error=False)
-        elif load_files and data_dir_empty:
-            self.show_message(_('The selected folder is empty. A new journal has been created.'),
-                                error=False)
-
+        logging.info('Opening journal at %r' % data_dir)
         self.dirs.data_dir = data_dir
 
         self.month = None
         self.months.clear()
 
-        # We always want to load all files
-        if load_files or True:
-            self.months = storage.load_all_months_from_disk(data_dir)
+        self.months = storage.load_all_months_from_disk(data_dir)
 
         # Nothing to save before first day change
         self.load_day(self.actual_date)
 
         self.stats = Statistics(self)
 
-        sorted_categories = sorted(self.node_names, key=lambda category: str(category).lower())
-        self.frame.categories_tree_view.categories = sorted_categories
-
-        if self.dirs.is_first_start and data_dir_empty:
-            logging.info('Adding example content')
+        if self.is_first_start and not os.listdir(data_dir) and len(self.days) == 0:
             self.add_instruction_content()
 
-        # Notebook is only on page 1 here, if we are opening a journal the second time
-        old_page = self.frame.search_notebook.get_current_page()
-        new_page = self.config.read('cloudTabActive', 1)
-        # 0 -> 0: search is cleared later
-        # 0 -> 1: change to cloud, update automatically
-        # 1 -> 0: change to search
-        # 1 -> 1: update cloud
-
-        # At tab change, cloud is updated automatically
-        self.frame.search_notebook.set_current_page(new_page)
-        if new_page == old_page:
-            # Without tab change, force update
-            self.frame.cloud.update(force_update=True)
+        self.frame.cloud.update(force_update=True)
 
         # Reset Search
         self.frame.search_box.clear()
 
+        self.frame.categories_tree_view.categories = self.categories
+        # Add auto-completion for tag search
+        self.frame.search_box.set_entries([u'#%s' % self.normalize_tag(tag)
+                                           for tag in self.categories])
+
         self.title = filesystem.get_journal_title(data_dir)
 
         # Set frame title
@@ -502,7 +416,7 @@ class Journal:
         self.day.content = self.frame.categories_tree_view.get_day_content()
         self.day.text = self.frame.get_day_text()
 
-        content_changed = not (old_content == self.day.content)
+        content_changed = (old_content != self.day.content)
         if content_changed:
             self.month.edited = True
 
@@ -533,9 +447,9 @@ class Journal:
             month.edited = True
 
 
-    def _get_current_day(self):
+    @property
+    def day(self):
         return self.month.get_day(self.date.day)
-    day = property(_get_current_day)
 
 
     def change_date(self, new_date):
@@ -547,53 +461,77 @@ class Journal:
 
 
     def go_to_next_day(self):
-        self.change_date(self.date + dates.one_day)
+        next_date = self.date + dates.one_day
+        following_edited_days = self.get_days_in_date_range(start_date=next_date)
+        if following_edited_days:
+            next_date = following_edited_days[0].date
+        self.change_date(next_date)
 
 
     def go_to_prev_day(self):
-        self.change_date(self.date - dates.one_day)
+        prev_date = self.date - dates.one_day
+        previous_edited_days = self.get_days_in_date_range(end_date=prev_date)
+        if previous_edited_days:
+            prev_date = previous_edited_days[-1].date
+        self.change_date(prev_date)
 
 
-    def show_message(self, message_text, error=False, countdown=True):
-        self.frame.statusbar.show_text(message_text, error, countdown)
+    def show_message(self, msg, title=None, error=False):
+        if error and not title:
+            title = _('Error')
+        msg_type = gtk.MESSAGE_ERROR if error else gtk.MESSAGE_INFO
+        self.frame.show_message(title, msg, msg_type)
 
 
     @property
-    def node_names(self):
-        node_names = set([])
-        for month in self.months.values():
-            node_names |= set(month.node_names)
-        return list(node_names)
+    def categories(self):
+        return list(sorted(set(itertools.chain.from_iterable(
+                                        day.categories for day in self.days)),
+                           cmp=locale.strcoll))
 
 
-    @property
-    def tags(self):
-        tags = set([])
-        for month in self.months.values():
-            tags |= set(month.tags)
-        return list(tags)
+    def normalize_tag(self, tag):
+        return tag.replace(' ', '').lower()
 
 
-    def search(self, text=None, category=None, tag=None):
-        results = []
+    def get_entries(self, category):
+        entries = set()
         for day in self.days:
-            result = None
-            if text:
-                result = day.search_text(text)
-            elif category:
-                result = day.search_category(category)
-            elif tag:
-                result = day.search_tag(tag)
+            entries |= set(day.get_entries(category))
+        return sorted(entries)
 
-            if result:
-                if category:
-                    results.extend(result)
-                else:
-                    results.append(result)
 
+    def search(self, text, tags):
+        days = self.get_days_with_tags(tags)
+        results = []
+        for day in reversed(days):
+            results.append(day.search(text, tags))
         return results
 
 
+    def get_days_with_tags(self, tags):
+        if not tags:
+            return self.days
+        days = []
+        for day in self.days:
+            day_tags = set(data.escape_tag(tag) for tag in day.categories)
+            if all(tag in day_tags for tag in tags):
+                days.append(day)
+        return days
+
+
+    def get_word_count_dict(self):
+        '''
+        Returns a dictionary mapping the words to their number of appearance
+        '''
+        word_dict = defaultdict(int)
+        for day in self.days:
+            words = day.get_words()
+            for word in words:
+                word_dict[word.lower()] += 1
+        return word_dict
+
+
     @property
     def days(self):
         '''
@@ -616,60 +554,32 @@ class Journal:
         return days
 
 
-    def get_word_count_dict(self, type):
-        '''
-        Returns a dictionary mapping the words to their number of appearance
-        '''
-        word_dict = collections.defaultdict(int)
-        for day in self.days:
-            if type == 'word':
-                words = day.words
-            if type == 'category':
-                words = day.node_names
-            if type == 'tag':
-                words = day.tags
+    def get_days_in_date_range(self, start_date=None, end_date=None):
+        if not start_date:
+            start_date = datetime.date.min
+        if not end_date:
+            end_date = datetime.date.max
 
-            for word in words:
-                word_dict[word.lower()] += 1
-        return word_dict
-
-
-    def get_days_in_date_range(self, range):
-        start_date, end_date = range
+        start_date, end_date = sorted([start_date, end_date])
         assert start_date <= end_date
 
-        sorted_days = self.days
         days_in_date_range = []
-        for day in sorted_days:
+        for day in self.days:
             if day.date < start_date:
                 continue
-            elif day.date >= start_date and day.date <= end_date:
+            elif start_date <= day.date <= end_date:
                 days_in_date_range.append(day)
             elif day.date > end_date:
                 break
         return days_in_date_range
 
 
-    def get_edit_date_of_entry_number(self, entry_number):
-        sorted_days = self.days
-        if len(sorted_days) == 0:
-            return datetime.date.today()
-        return sorted_days[entry_number % len(sorted_days)].date
-
-
-    def go_to_first_empty_day(self):
-        if len(self.days) == 0:
-            return datetime.date.today()
-
-        last_edited_day = self.days[-1]
-        first_empty_date = last_edited_day.date + dates.one_day
-        self.change_date(first_empty_date)
-
-
     def add_instruction_content(self):
-        self.go_to_first_empty_day()
+        self.change_date(datetime.date.today())
         current_date = self.date
 
+        logging.info('Adding example content on %s' % current_date)
+
         for example_day in info.example_content:
             self.day.content = example_day
             self.frame.set_date(self.month, self.date, self.day)
@@ -689,8 +599,6 @@ def main():
     try:
         logging.debug('Trying to enter the gtk main loop')
         gtk.main()
-        #logging.debug('Closing logfile')
-        #file_logging_stream.close()
     except KeyboardInterrupt:
         # 'Interrupt'
         #journal.save_to_disk()
