@@ -17,9 +17,6 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-# For Python 2.5 compatibility.
-from __future__ import with_statement
-
 import os
 import sys
 import logging
@@ -35,8 +32,7 @@ except ImportError:
 # The presence of the yaml module has been checked
 try:
     from yaml import CLoader as Loader
-    from yaml import CDumper as Dumper
-    #logging.info('Using libyaml for loading and dumping')
+    from yaml import CSafeDumper as Dumper
 except ImportError:
     from yaml import Loader, Dumper
     logging.info('Using pyyaml for loading and dumping')
@@ -44,6 +40,22 @@ except ImportError:
 from rednotebook.data import Month
 
 
+def get_journal_files(data_dir):
+    # Format: 2010-05.txt
+    date_exp = re.compile(r'(\d{4})-(\d{2})\.txt$')
+
+    for file in sorted(os.listdir(data_dir)):
+        match = date_exp.match(file)
+        if match:
+            year = int(match.group(1))
+            month = int(match.group(2))
+            assert month in range(1, 12 + 1)
+            path = os.path.join(data_dir, file)
+            yield (path, year, month)
+        else:
+            logging.debug('%s is not a valid month filename' % file)
+
+
 def _load_month_from_disk(path, year_number, month_number):
     '''
     Load the month file at path and return a month object
@@ -53,9 +65,8 @@ def _load_month_from_disk(path, year_number, month_number):
     try:
         # Try to read the contents of the file
         with codecs.open(path, 'rb', encoding='utf-8') as month_file:
-            logging.debug('Start loading file "%s"' % path)
+            logging.debug('Loading file "%s"' % path)
             month_contents = yaml.load(month_file, Loader=Loader)
-            logging.debug('Finished loading file "%s"' % path)
             month = Month(year_number, month_number, month_contents)
             return month
     except yaml.YAMLError, exc:
@@ -65,39 +76,24 @@ def _load_month_from_disk(path, year_number, month_number):
         logging.error('Error: The file %s could not be read' % path)
     except Exception, err:
         logging.error('An error occured while reading %s:' % path)
-        logging.error('%s' % err)
+        raise
+    # If we continued here, the possibly corrupted file would be overwritten
+    sys.exit(1)
 
 
 def load_all_months_from_disk(data_dir):
     '''
     Load all months and return a directory mapping year-month values
-    to month objects
+    to month objects.
     '''
-    # Format: 2010-05.txt
-    date_exp = re.compile(r'(\d{4})-(\d{2})\.txt$')
-
     months = {}
 
     logging.debug('Starting to load files in dir "%s"' % data_dir)
-    files = sorted(os.listdir(data_dir))
-    for file in files:
-        match = date_exp.match(file)
-        if match:
-            year_string = match.group(1)
-            month_string = match.group(2)
-            year_month = year_string + '-' + month_string
+    for path, year_number, month_number in get_journal_files(data_dir):
+        month = _load_month_from_disk(path, year_number, month_number)
+        if month:
+            months['%d-%02d' % (year_number, month_number)] = month
 
-            year_number = int(year_string)
-            month_number = int(month_string)
-            assert month_number in range(1, 13)
-
-            path = os.path.join(data_dir, file)
-
-            month = _load_month_from_disk(path, year_number, month_number)
-            if month:
-                months[year_month] = month
-        else:
-            logging.debug('%s is not a valid month filename' % file)
     logging.debug('Finished loading files in dir "%s"' % data_dir)
     return months
 
@@ -112,20 +108,24 @@ def save_months_to_disk(months, dir, frame, exit_imminent=False, changing_journa
         if month.edited or saveas:
             something_saved = True
             month_file_string = os.path.join(dir, year_and_month + '.txt')
-            with codecs.open(month_file_string, 'wb', encoding='utf-8') as month_file:
-                month_content = {}
-                for day_number, day in month.days.iteritems():
-                    # do not add empty days
-                    if not day.empty:
-                        month_content[day_number] = day.content
+            month_content = {}
+            for day_number, day in month.days.iteritems():
+                # do not add empty days
+                if not day.empty:
+                    month_content[day_number] = day.content
 
+            # Do not save empty month files
+            if not month_content and not os.path.exists(month_file_string):
+                continue
+
+            with codecs.open(month_file_string, 'wb', encoding='utf-8') as month_file:
                 try:
-                    # yaml.dump(month_content, month_file, Dumper=Dumper)
                     # This version produces readable unicode and no python directives
-                    yaml.safe_dump(month_content, month_file, allow_unicode=True)
+                    yaml.dump(month_content, month_file, Dumper=Dumper, allow_unicode=True)
                     month.edited = False
-                except OSError, err:
+                    logging.debug('Wrote file %s' % month_file_string)
+                except OSError:
                     frame.show_save_error_dialog(exit_imminent)
-                except IOError, err:
+                except IOError:
                     frame.show_save_error_dialog(exit_imminent)
     return something_saved
