@@ -17,21 +17,17 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import os
-import sys
-import logging
-import time
-import codecs
 
 import gtk
 
-
 from rednotebook.util import filesystem
 from rednotebook.util import dates
 
 
+WEEKDAYS = (_('Monday'), _('Tuesday'), _('Wednesday'), _('Thursday'),
+            _('Friday'), _('Saturday'), _('Sunday'))
+
 
 example_text = '''\
 === This is an example template ===
@@ -49,6 +45,7 @@ Your text can be:
 - //italic//
 - __underlined__
 - --strikethrough--
+- or some **//__combination__//**
 
 
 You can add images to your template:
@@ -67,14 +64,25 @@ empty lines to end a list.
 
 Additionally you can have **titles** and **horizontal lines**:
 
-===Title===
+= Title level 1 =
+(The dates in the export will use this level, so it is recommended to use lower
+levels in your entries)
+== Title level 2 ==
+=== Title level 3 ===
+etc.
 
 ====================
 
+% Commentary text can be put on lines starting with a percent sign.
+% Those lines will not show up in the preview and the export.
+
 **Macros**:
 
 When a template is inserted, every occurence of "$date$" is converted to \
 the current date. You can set the date format in the preferences.
+
+There is even more markup that you can put into your templates. Have a look at
+the inline help (Ctrl+H) for information.
 '''
 
 help_text = '''\
@@ -82,20 +90,11 @@ Besides templates for weekdays you can also have arbitrary named templates.
 For example you might want to have a template for "Meeting" or "Journey".
 All templates must reside in the directory "%s".
 
-To create a new template, just save an ordinary textfile in that directory. \
-You can use your favourite text editing program for that task \
-(e.g. gedit on Linux or the editor on Win).
-The name of the newly created file will be the template's title.
-
-You can switch to "Preview" mode and click on the link to get to the \
-[template directory ""%s""].
-
-If you come up with templates that could be useful for other people as well, \
-I would appreciate if you sent me your template file, so others can benefit \
-from it.
+The template button gives you the options to create a new template or to \
+visit the templates directory.
 '''
 
-meeting = '''\
+meeting = _('''\
 === Meeting ===
 
 Purpose, date, and place
@@ -117,9 +116,9 @@ Purpose, date, and place
 +
 +
 ==================================
-'''
+''')
 
-journey = '''\
+journey = _('''\
 === Journey ===
 **Date:**
 
@@ -131,9 +130,9 @@ journey = '''\
 First we went to xxxxx then we got to yyyyy ...
 
 **Pictures:** [Image folder ""/path/to/the/images/""]
-'''
+''')
 
-call = '''\
+call = _('''\
 ==================================
 === Phone Call ===
 - **Person:**
@@ -141,9 +140,9 @@ call = '''\
 - **Topic:**
 - **Outcome and Follow up:**
 ==================================
-'''
+''')
 
-personal = '''\
+personal = _('''\
 =====================================
 === Personal ===
 
@@ -161,7 +160,7 @@ personal = '''\
 +
 +
 =====================================
-'''
+''')
 
 
 class TemplateManager(object):
@@ -174,7 +173,7 @@ class TemplateManager(object):
         self.actiongroup = None
 
     def on_insert(self, action):
-        title = action.get_name()
+        title = unicode(action.get_name())
 
         # strip 'Insert'
         title = title[6:]
@@ -189,7 +188,7 @@ class TemplateManager(object):
         '''
         Open the template file in an editor
         '''
-        edit_title = action.get_name()
+        edit_title = unicode(action.get_name())
         title = edit_title[4:]
 
         if title == 'Weekday':
@@ -273,62 +272,59 @@ class TemplateManager(object):
         See http://www.pygtk.org/pygtk2tutorial/sec-UIManager.html for help
         A popup menu cannot show accelerators (HIG).
         '''
-
         # complete paths
         files = self.get_available_template_files()
 
+        def escape_name(name):
+            """Remove special xml chars for GUI display."""
+            for char in '&<>\'"':
+                name = name.replace(char, '')
+            return name
+
         # 1, 2, 3
         self.titles_to_files = {}
         for file in files:
             root, ext = os.path.splitext(file)
             title = os.path.basename(root)
-            self.titles_to_files[title] = file
+            self.titles_to_files[escape_name(title)] = file
 
         sorted_titles = sorted(self.titles_to_files.keys())
 
         menu_xml = '''\
         <ui>
-        <popup action="TemplateMenu">
-        '''
+        <popup action="TemplateMenu">'''
 
-        insert_menu_xml = '''\
+        insert_menu_xml = '''
             <menu action="InsertMenu">
                 <menuitem action="InsertWeekday"/>
-                <separator name="sep4"/>
-        '''
+                <separator name="sep4"/>'''
         for title in sorted_titles:
-            if title not in map(str, range(1,8)):
-                insert_menu_xml += '''\
-                <menuitem action="Insert%s"/>
-                ''' % title
-        insert_menu_xml += '''\
-            </menu>
-        '''
+            if title not in map(str, range(1, 8)):
+                insert_menu_xml += '''
+                <menuitem action="Insert%s"/>''' % title
+        insert_menu_xml += '''
+            </menu>'''
 
         menu_xml += insert_menu_xml
 
-        menu_xml += '''\
+        menu_xml += '''
             <separator name="sep5"/>
-            <menuitem action="NewTemplate"/>
-        '''
+            <menuitem action="NewTemplate"/>'''
 
-        edit_menu_xml = '''\
+        edit_menu_xml = '''
             <menu action="EditMenu">
                 <menuitem action="EditWeekday"/>
-                <separator name="sep3"/>
-        '''
+                <separator name="sep3"/>'''
         for title in sorted_titles:
             if title not in map(str, range(1,8)):
-                edit_menu_xml += '''\
-                <menuitem action="Edit%s"/>
-                ''' % title
-        edit_menu_xml += '''\
-            </menu>
-        '''
+                edit_menu_xml += '''
+                <menuitem action="Edit%s"/>''' % title
+        edit_menu_xml += '''
+            </menu>'''
 
         menu_xml += edit_menu_xml
 
-        menu_xml +='''\
+        menu_xml +='''
             <menuitem action="OpenTemplateDirectory"/>
         </popup>
         </ui>'''
@@ -393,21 +389,21 @@ class TemplateManager(object):
 
         files = []
         for day_number in range(1, 8):
-            files.append((self.get_template_file(day_number), example_text))
+            weekday = WEEKDAYS[day_number - 1]
+            files.append((self.get_template_file(day_number),
+                          example_text.replace('template ===',
+                                               'template for %s ===' % weekday)))
 
-        help_text %= (self.dirs.template_dir, self.dirs.template_dir)
+        help_text %= (self.dirs.template_dir)
 
         files.append((self.get_template_file('Help'), help_text))
 
         # Only add the example templates the first time and just restore
         # the day templates everytime
-        if not self.main_window.journal.dirs.is_first_start:
-            filesystem.make_files(files)
-            return
-
-        files.append((self.get_template_file('Meeting'), meeting))
-        files.append((self.get_template_file('Journey'), journey))
-        files.append((self.get_template_file('Call'), call))
-        files.append((self.get_template_file('Personal'), personal))
+        if self.main_window.journal.is_first_start:
+            files.append((self.get_template_file('Meeting'), meeting))
+            files.append((self.get_template_file('Journey'), journey))
+            files.append((self.get_template_file('Call'), call))
+            files.append((self.get_template_file('Personal'), personal))
 
         filesystem.make_files(files)
