@@ -17,8 +17,6 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import os
 import imp
 import sys
@@ -52,18 +50,19 @@ def get_main_dir():
 #------------------------------------------------------------------------------
 
 
-if not main_is_frozen():
+if main_is_frozen():
+    app_dir = get_main_dir()
+    locale_dir = os.path.join(app_dir, 'share', 'locale')
+else:
     app_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../'))
     app_dir = os.path.normpath(app_dir)
-else:
-    app_dir = get_main_dir()
+    locale_dir = os.path.join(sys.prefix, 'share', 'locale')
 
 app_dir = get_unicode_path(app_dir)
 
 image_dir = os.path.join(app_dir, 'images')
 frame_icon_dir = os.path.join(image_dir, 'rednotebook-icon')
 files_dir = os.path.join(app_dir, 'files')
-locale_dir = os.path.join(app_dir, 'i18n')
 
 user_home_dir = get_unicode_path(os.path.expanduser('~'))
 
@@ -94,6 +93,8 @@ class Filenames(dict):
         self.last_pic_dir = self.user_home_dir
         self.last_file_dir = self.user_home_dir
 
+        self.forbidden_dirs = [user_home_dir, self.journal_user_dir]
+
 
     def get_user_dir(self, config):
         custom = config.read('userDir', '')
@@ -114,6 +115,10 @@ class Filenames(dict):
         return user_dir
 
 
+    def is_valid_journal_path(self, path):
+        return os.path.isdir(path) and os.path.abspath(path) not in self.forbidden_dirs
+
+
     def __getattribute__(self, attr):
         user_paths = dict((('template_dir', 'templates'),
                         ('temp_dir', 'tmp'),
@@ -222,9 +227,6 @@ def get_relative_path(from_dir, to_dir):
     else:
         return to_dir
 
-def get_icons():
-    return glob(os.path.join(frame_icon_dir, '*.png'))
-
 
 def get_journal_title(dir):
     '''
@@ -246,9 +248,7 @@ def get_platform_info():
 
     functions = [platform.machine, platform.platform, platform.processor,
                 platform.python_version, platform.release, platform.system,]
-    values = map(lambda function: function(), functions)
-    functions = map(lambda function: function.__name__, functions)
-    names_values = zip(functions, values)
+    names_values = [(func.__name__, func()) for func in functions]
 
     lib_values = [('GTK version', gtk, 'gtk_version'),
                     ('PyGTK version', gtk, 'pygtk_version'),
@@ -308,6 +308,15 @@ def unquote_url(url):
     return urllib.unquote(url).decode('utf-8')
 
 
+def _open_url_with_call(url, prog):
+    try:
+        logging.info( 'Trying to open %s with %s' % (url, prog))
+        system_call([prog, url])
+    except (OSError, subprocess.CalledProcessError):
+        logging.exception('Opening %s with %s failed' % (url, prog))
+        # If everything failed, try the webbrowser
+        open_url_in_browser(url)
+
 def open_url(url):
     '''
     Opens a file with the platform's preferred method
@@ -325,29 +334,12 @@ def open_url(url):
             logging.info('Trying to open %s with "os.startfile"' % url)
             # os.startfile is only available on windows
             os.startfile(url)
-            return
         except OSError:
             logging.exception('Opening %s with "os.startfile" failed' % url)
-            return
-
     elif sys.platform == 'darwin':
-        try:
-            logging.info('Trying to open %s with "open"' % url)
-            system_call(['open', url])
-            return
-        except (OSError, subprocess.CalledProcessError):
-            logging.exception('Opening %s with "open" failed' % url)
-
+        _open_url_with_call(url, 'open')
     else:
-        try:
-            logging.info( 'Trying to open %s with xdg-open' % url)
-            system_call(['xdg-open', url])
-            return
-        except (OSError, subprocess.CalledProcessError):
-            logging.exception('Opening %s with xdg-open failed' % url)
-
-    # If everything failed, try the webbrowser
-    open_url_in_browser(url)
+        _open_url_with_call(url, 'xdg-open')
 
 
 
